//+---------------------------------------------------------------------------------+
//|                                             					 EA_IWBDT.mq5   |
//|                                                                                 |
//|                          Copyright © 2023, Casper and Sharky, The Ghost Project |
//|                                                  https://www.Coming_One_Day.com |
//+---------------------------------------------------------------------------------+

#property copyright "Casper & Sharktoshi, The Ghost Project"
#property link      "Coming_One_Day.com"
#ifndef INCLUDES
	#define INCLUDES
		#include <Trade/OrderInfo.mqh>
		COrderInfo orderInfo;

		#include <Trade/PositionInfo.mqh>
		CPositionInfo positionInfo;

		#include <Trade/Pending.mqh>
		CPending pendingOrders;

		// Price
		#include <Trade/Price.mqh>
		CBars Bar;

		// Trailing stops
		#include <Trade/TrailingStops.mqh>
		CTrailing Trail;

		// Indicators
		#include <Trade/Indicators.mqh>
		CiMA FastMovingAverageDaily;
		CiMA FastMovingAverage4HR;
		CiMA FastMovingAverage1HR;
		CiMA SlowMovingAverageDaily;
		CiMA SlowMovingAverage4HR;
		CiMA SlowMovingAverage1HR;
		CiMA TrailingStopFastMovingAverage;
		CiMA TrailingStopSlowMovingAverage;

		//Trading Utilities
		#include <Ghost Project\TradingUtilities.mqh>
		CTradingUtilities TradingUtilities;

		#include <Trade/GhostTrade.mqh>
		CGhostTrade Trade;
#endif

#define MAX_RETRIES 5		// Max retries on error
#define RETRY_DELAY 3000	// Retry delay in ms

//+------------------------------------------------------------------+
//| Input variables                                                  |
//+------------------------------------------------------------------+
int lot = 2; // Contracts Per Trade


int inRiskReward = 2; // Risk to Reward Amount. Default is 2:1 RR
int inMaximumRiskPerTrade = 45; // Risk per trade. Default is $5 risk to win $10
ulong inSlippage = 0; // TODO: During backtesting/forward testing dig more into this potential item. This may be able to be an int. 2023-08-01 12:47:17


bool inTradeOnNewBar = false; // Make true if some action needs to happen on each new bar. C - 2023-08-21

// Prob wont use below until much later when we incorporate money management -- C 2023-08-20
bool inUseMoneyManagement = false; //Make true when ready to use Money management to dictate when trades can be taken. C - 2023-08-21

// double riskPercent = 2; // wont be used right away -- C 2023-08-20
double inFixedVolume = 2;

// Prob wont use below. StopLoss and TakeProfit are ints that are used to say how many points we want our stop or tp to be from entry -- C 2023-08-20
int inStopLossInPoints = 0;
int inTakeProfitInPoints = 0;

bool inUseTrailingStop = true;

bool inUseBreakEven = false;

// Fast MA Data
int inFastMovingAveragePeriod = 9;
ENUM_MA_METHOD inFastMovingAverageMode = MODE_EMA;
int inFastMovingAverageBarShift = 1; // @deprecated TODO: If still exists unused at final refactoring, remove. C-2023-08-21
ENUM_APPLIED_PRICE inFastMovingAverageAppliedPrice = PRICE_CLOSE;

// Slow MA Data
int inSlowMovingAveragePeriod = 18;
ENUM_MA_METHOD inSlowMovingAverageMode = MODE_EMA;
int inSlowMovingAverageBarShift = 1; // @deprecated TODO: If still exists unused at final refactoring, remove. C-2023-08-21
ENUM_APPLIED_PRICE inSlowMovingAverageAppliedPrice = PRICE_CLOSE;

// Trailing Stop Fast MA Data
int inTrailingStopFastMovingAveragePeriod = 13;
ENUM_MA_METHOD inTrailingStopFastMovingAverageMode = MODE_EMA;
int inTrailingStopFastMovingAverageBarShift = 1; // @deprecated TODO: If still exists unused at final refactoring, remove. C-2023-08-21
ENUM_APPLIED_PRICE inTrailingStopFastMovingAverageAppliedPrice = PRICE_CLOSE;

// Trailing Stop Slow MA Data
int inTrailingStopSlowMovingAveragePeriod = 30;
ENUM_MA_METHOD inTrailingStopSlowMovingAverageMode = MODE_SMA;
int inTrailingStopSlowMovingAverageBarShift = 1; // @deprecated TODO: If still exists unused at final refactoring, remove. C-2023-08-21
ENUM_APPLIED_PRICE inTrailingStopSlowMovingAverageAppliedPrice = PRICE_CLOSE;

// Timer
bool inUseTimer = true;
int inStartHour = 15;
int inStartMinute = 0;
int inEndHour = 13;
int inEndMinute = 0;
bool inUseLocalTime = true;

// Num Bars for Fractals
bool inUseBarFractals = false; // @deprecated TODO: If still exists unused at final refactoring, remove. C-2023-10-24
int inNumBarsForFractals = 1;
bool inHaramiStrictEnabled = true;
bool inHaramiEnabled = true;


// debug print
bool inUseDebugPrint = false;

//+------------------------------------------------------------------+
//| Global variables                                                 |
//+------------------------------------------------------------------+
bool glBuyPlaced = false;
bool glSellPlaced = false;
bool glValidSetup = false;

bool glBuyEnabled = true;
bool glSellEnabled = true;

int glBarShift = 1;
int glNumContracts = 2;

// Moving Average Handles
int glFastMovingAverageDailyHandle;
int glFastMovingAverage4HRHandle;
int glFastMovingAverage1HRHandle;

int glSlowMovingAverageDailyHandle;
int glSlowMovingAverage4HRHandle;
int glSlowMovingAverage1HRHandle;

int glTrailingStopFastMovingAverageHandle;
int glTrailingStopSlowMovingAverageHandle;

enum Signal
{
	SIGNAL_BUY,
	SIGNAL_SELL,
	SIGNAL_NONE,
};
Signal glSignal;

// RSI Global Variables
int RSIPeriod      = 14; // RSI period
ENUM_APPLIED_PRICE    Inp_RSI_applied_price   = PRICE_CLOSE; // Applied price
double    rsiBuffer[];
double    highestRSIBuffer[];
double    lowestRSIBuffer[];
int rsiHandle = INVALID_HANDLE;    // variable for storing the handle of the iRSI indicator
int barsCalculated = 0;            // we will keep the number of values in the RSI indicator
bool initError = false;            // error OnInit


//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit(){
	glFastMovingAverageDailyHandle = FastMovingAverageDaily.Init(_Symbol, PERIOD_D1, inFastMovingAveragePeriod, glBarShift, inFastMovingAverageMode, inFastMovingAverageAppliedPrice);
	glFastMovingAverage4HRHandle = FastMovingAverage4HR.Init(_Symbol, PERIOD_H4, inFastMovingAveragePeriod, glBarShift, inFastMovingAverageMode, inFastMovingAverageAppliedPrice);
	glFastMovingAverage1HRHandle = FastMovingAverage1HR.Init(_Symbol, PERIOD_H1, inFastMovingAveragePeriod, glBarShift, inFastMovingAverageMode, inFastMovingAverageAppliedPrice);

	glSlowMovingAverageDailyHandle = SlowMovingAverageDaily.Init(_Symbol, PERIOD_D1, inSlowMovingAveragePeriod, glBarShift, inSlowMovingAverageMode, inSlowMovingAverageAppliedPrice);
	glSlowMovingAverage4HRHandle = SlowMovingAverage4HR.Init(_Symbol, PERIOD_H4, inSlowMovingAveragePeriod, glBarShift, inSlowMovingAverageMode, inSlowMovingAverageAppliedPrice);
	glSlowMovingAverage1HRHandle = SlowMovingAverage1HR.Init(_Symbol, PERIOD_H1, inSlowMovingAveragePeriod, glBarShift, inSlowMovingAverageMode, inSlowMovingAverageAppliedPrice);

	glTrailingStopFastMovingAverageHandle = TrailingStopFastMovingAverage.Init(_Symbol, _Period, inTrailingStopFastMovingAveragePeriod, glBarShift, inTrailingStopFastMovingAverageMode, inFastMovingAverageAppliedPrice);
	glTrailingStopSlowMovingAverageHandle = TrailingStopSlowMovingAverage.Init(_Symbol, _Period, inTrailingStopSlowMovingAveragePeriod, glBarShift, inTrailingStopSlowMovingAverageMode, inTrailingStopSlowMovingAverageAppliedPrice);

	Trade.Deviation(inSlippage);

	//--- create handle of the iRSI indicator
    rsiHandle = iRSI(Symbol(), Period(), RSIPeriod, Inp_RSI_applied_price);

    //--- if the handle is not created
    if(rsiHandle == INVALID_HANDLE)
    {
        //--- tell about the failure and output the error code
        PrintFormat("Failed to create handle of the iRSI indicator for the symbol %s/%s, error code %d",
            Symbol(),
            EnumToString(Period()),
            GetLastError());
        //--- the indicator is stopped early
        initError = true;
        return(INIT_FAILED);
    }

	return(INIT_SUCCEEDED);
}

//TODO: Need to update this to use indiactors.release() - C-2023-08-22
int OnDeInit(const int reason){
	IndicatorRelease(glFastMovingAverageDailyHandle);
	IndicatorRelease(glFastMovingAverage4HRHandle);
	IndicatorRelease(glFastMovingAverage1HRHandle);
	IndicatorRelease(glSlowMovingAverageDailyHandle);
	IndicatorRelease(glSlowMovingAverage4HRHandle);
	IndicatorRelease(glSlowMovingAverage1HRHandle);
	IndicatorRelease(glTrailingStopFastMovingAverageHandle);
	IndicatorRelease(glTrailingStopSlowMovingAverageHandle);
	return(0); // should this be something else like true or false or maybe -1? - C-2023-08-21
}

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick(){
	// Check for new bar
	bool newBar = false;
	bool activePosition = (PositionsTotal() > 0);
	bool activeOrders = (OrdersTotal() > 0);
	bool stopLossLevelHit = Trade.GetStopLossLevelHit();
	bool stopToScratchLevelHit = Trade.GetStopToScratchLevelHit();
	bool stopToTrailingLevelHit = Trade.GetStopToTrailingLevelHit();
	bool tradeLevelsValidated = Trade.GetTradeLevelsValidated();
	bool takeProfitLevelHit = Trade.GetTakeProfitLevelHit();

	// Normalize entry and stop loss levels
	// TODO: Make sure these are reset before use - C-2023-08-24
	double normalizedEntryLevel = Trade.GetNormalizedEntryLevel();
	double normalizedStopLossLevel = Trade.GetNormalizedStopLossLevel();
	double normalizedTakeProfitLevel = Trade.GetNormalizedTakeProfitLevel();
	double normalizedStopToScratchLevel = Trade.GetNormalizedStopToScratchLevel();
	double normalizedStopToTrailingLevel = Trade.GetNormalizedStopToTrailingLevel();
	double currentAsk = Bar.Ask(0);
	double currentBid = Bar.Bid(0);

	// Trending MAs check
	bool trendingBullishEMAs = false;
	bool trendingBearishEMAs = false;

	// Price fractal checks
	bool bullishPriceFractal = false;
	bool bearishPriceFractal = false;

	double entryLevel = 0.0;
	double stopLossLevel = 0.0;
	double symbolStopsLevel = TradingUtilities.DetermineStopsLevelForSymbol(_Symbol); //TODO: Can a symbols "stops level" change? If not, this can be done as a global C-09-08-2023

	//Not needed but it does clean up the look of the calls to DetectBuillishTrendingEMAs and the bear version thereof. C-2023-08-21
	double fastMovingAverageHTF = FastMovingAverageDaily.Main(glBarShift);
	double slowMovingAverageHTF = SlowMovingAverageDaily.Main(glBarShift);
	double fastMovingAverageMTF = FastMovingAverage4HR.Main(glBarShift);
	double slowMovingAverageMTF = SlowMovingAverage4HR.Main(glBarShift);
	double fastMovingAverageLTF = FastMovingAverage1HR.Main(glBarShift);
	double slowMovingAverageLTF = SlowMovingAverage1HR.Main(glBarShift);

	// Reset important variables after trade is complete.
	if(!activeOrders && !activePosition && (glBuyPlaced || glSellPlaced)){		
		glBuyPlaced = false;
		glSellPlaced = false;
		glValidSetup = false;
		Trade = new CGhostTrade();
	}

	// Update prices
	Bar.Update(_Symbol,_Period);

	// Bull/Bear trending MAs check
	if(!activeOrders && !activePosition){
		trendingBullishEMAs = TradingUtilities.DetectBullishTrendingEMAs(fastMovingAverageHTF, slowMovingAverageHTF, fastMovingAverageMTF, slowMovingAverageMTF, fastMovingAverageLTF, slowMovingAverageLTF);
		trendingBearishEMAs = TradingUtilities.DetectBearishTrendingEMAs(fastMovingAverageHTF, slowMovingAverageHTF, fastMovingAverageMTF, slowMovingAverageMTF, fastMovingAverageLTF, slowMovingAverageLTF);
	}

	// Bull Price Fractal Check
	if(trendingBullishEMAs && !activeOrders && !activePosition){
		if(inNumBarsForFractals == 5){
			bullishPriceFractal = TradingUtilities.DetectBullish5BarPriceFractal(Bar.Low(1), Bar.Low(2), Bar.Low(3), Bar.Low(4), Bar.Low(5));
			entryLevel = Bar.High(1);
			stopLossLevel = Bar.Low(3);
		}else if(inNumBarsForFractals == 3){
			bullishPriceFractal = TradingUtilities.DetectBullish3BarPriceFractal(Bar.Low(1), Bar.Low(2), Bar.Low(3));
			entryLevel = Bar.High(1);
			stopLossLevel = Bar.Low(2);
		}else if(inHaramiEnabled){
			if(inHaramiStrictEnabled){
				bullishPriceFractal = TradingUtilities.DetectBullishHaramiStrict(Bar.High(1), Bar.Low(1), Bar.Open(1), Bar.Close(1), Bar.Open(2), Bar.Close(2));
			}else{
				bullishPriceFractal = TradingUtilities.DetectBullishHarami(Bar.High(1), Bar.Low(1), Bar.Open(1), Bar.Close(1), Bar.High(2), Bar.Low(2));
			}
			entryLevel = Bar.High(1);
			stopLossLevel = Bar.Low(2);
		}else{
			string debugPrintString = "Error: Invalid number of bars for fractals. Must be 3 or 5.";
			TradingUtilities.DebugPrint(debugPrintString);
		}
	}

	// Bear Price Fractal Check
	if(trendingBearishEMAs && !activeOrders && !activePosition){
		if(inNumBarsForFractals == 5){
			bearishPriceFractal = TradingUtilities.DetectBearish5BarPriceFractal(Bar.High(1), Bar.High(2), Bar.High(3), Bar.High(4), Bar.High(5));
			entryLevel = Bar.Low(1);
			stopLossLevel = Bar.High(3);
		}else if(inNumBarsForFractals == 3){
			bearishPriceFractal = TradingUtilities.DetectBearish3BarPriceFractal(Bar.High(1), Bar.High(2), Bar.High(3));
			entryLevel = Bar.Low(1);
			stopLossLevel = Bar.High(2);
		}else if(inHaramiEnabled){
			if(inHaramiStrictEnabled){
				bearishPriceFractal = TradingUtilities.DetectBearishHaramiStrict(Bar.High(1), Bar.Low(1), Bar.Open(1), Bar.Close(1), Bar.Open(2), Bar.Close(2));
			}else{
				bearishPriceFractal = TradingUtilities.DetectBearishHarami(Bar.High(1), Bar.Low(1), Bar.Open(1), Bar.Close(1), Bar.High(2), Bar.Low(2));
			}
			entryLevel = Bar.Low(1);
			stopLossLevel = Bar.High(2);
		}else{
			string debugPrintString = "Error: Invalid number of bars for fractals. Must be 3 or 5.";
			TradingUtilities.DebugPrint(debugPrintString);
		}
	}

	//Bull Open Trade
	//Print("1 inBuyEnbled: " + glBuyEnabled + " glSellEnabled: " + glSellEnabled);
	if(glBuyEnabled && !activePosition && !activeOrders && bullishPriceFractal){
		bool success = false;
		normalizedEntryLevel = TradingUtilities.CalculateBullishEntryLevel(entryLevel);
		normalizedStopLossLevel = TradingUtilities.CalculateBullishStopLossLevel(stopLossLevel);
		normalizedTakeProfitLevel = TradingUtilities.CalculateBullishTakeProfitLevel(normalizedEntryLevel, normalizedStopLossLevel);
		normalizedStopToScratchLevel = TradingUtilities.CalculateBullishTakeProfitLevel(normalizedEntryLevel, normalizedStopLossLevel, 1);
		normalizedStopToTrailingLevel = normalizedTakeProfitLevel;

		Trade.SetNormalizedEntryLevel(normalizedEntryLevel);
		Trade.SetNormalizedStopLossLevel(normalizedStopLossLevel);
		Trade.SetNormalizedTakeProfitLevel(normalizedTakeProfitLevel);
		Trade.SetNormalizedStopToScratchLevel(normalizedStopToScratchLevel);
		Trade.SetNormalizedStopToTrailingLevel(normalizedStopToTrailingLevel);

		success = Trade.BuyStopWithValidation(_Symbol, glNumContracts, normalizedEntryLevel, normalizedStopLossLevel, normalizedTakeProfitLevel, normalizedStopToScratchLevel, normalizedStopToTrailingLevel, 0, NULL);
		if(success){
			glBuyPlaced = true;
			glValidSetup = true;
			tradeLevelsValidated = true;
			activePosition = false;
			activeOrders = true;
		}
		// TODO: Need to add detailed error handling/logging here to determine which level was not valid. C-2023-08-26 -- Is this now taken care of in the validate functions? C-2023-09-08
	}

	// Bear Open Trade
	if(glSellEnabled && !activePosition && !activeOrders && bearishPriceFractal){
		bool success = false;
		normalizedEntryLevel = TradingUtilities.CalculateBearishEntryLevel(entryLevel);
		normalizedStopLossLevel = TradingUtilities.CalculateBearishStopLossLevel(stopLossLevel);
		normalizedTakeProfitLevel = TradingUtilities.CalculateBearishTakeProfitLevel(normalizedEntryLevel, normalizedStopLossLevel);
		normalizedStopToScratchLevel = TradingUtilities.CalculateBearishTakeProfitLevel(normalizedEntryLevel, normalizedStopLossLevel, 1);
		normalizedStopToTrailingLevel = normalizedTakeProfitLevel;

		Trade.SetNormalizedEntryLevel(normalizedEntryLevel);
		Trade.SetNormalizedStopLossLevel(normalizedStopLossLevel);
		Trade.SetNormalizedTakeProfitLevel(normalizedTakeProfitLevel);
		Trade.SetNormalizedStopToScratchLevel(normalizedStopToScratchLevel);
		Trade.SetNormalizedStopToTrailingLevel(normalizedStopToTrailingLevel);

		success = Trade.SellStopWithValidation(_Symbol, glNumContracts, normalizedEntryLevel, normalizedStopLossLevel, normalizedTakeProfitLevel, normalizedStopToScratchLevel, normalizedStopToTrailingLevel, 0, NULL);
		if(success){
			glSellPlaced = true;
			glValidSetup = true;
			tradeLevelsValidated = true;
			activePosition = false;
			activeOrders = true;
		}
	}
	
	// Bull check to see if the trade has gone to the TP or SL levels before firing a position.
	if(activeOrders && !activePosition && glBuyPlaced){
		if(Bar.Low(0) <= normalizedStopLossLevel || Bar.High(0) >= normalizedTakeProfitLevel){
			bool success = Trade.DeleteAllPendingOrders(_Symbol);
			if(success){
				activePosition = false; // necessary so that stoptoscratch and trailing do not fire after we have invalidated the setup. C-2023-08-28
				glBuyPlaced = false;
			}
		}
	}
	
	// Bear check to see if the trade has gone to the TP or SL levels before firing a position.
	if(activeOrders && !activePosition && glSellPlaced){
		if(Bar.High(0) >= normalizedStopLossLevel || Bar.Low(0) <= normalizedTakeProfitLevel){
			bool success = Trade.DeleteAllPendingOrders(_Symbol);
			if(success){
				activePosition = false; // necessary so that stoptoscratch and trailing do not fire after we have invalidated the setup. C-2023-08-28
				glSellPlaced = false;
			}
		}
	}

	// Bull Check Entry Level Hit
	if(activePosition && glBuyPlaced && !Trade.GetEntryLevelHit()){ // && (Bar.Low(0) <= normalizedEntryLevel || Bar.High(0) >= normalizedEntryLevel)){
		Trade.SetEntryLevelHit(true);
	}

	// Bear Check Entry Level Hit C-2023-09-08
	if(activePosition && glSellPlaced && !Trade.GetEntryLevelHit()){ // && (Bar.Low(0) <= normalizedEntryLevel || Bar.High(0) >= normalizedEntryLevel)){
		Trade.SetEntryLevelHit(true);
	}

	// Bull Check Take Profit Level Hit
	if(activePosition && glBuyPlaced && (Bar.High(0) >= normalizedTakeProfitLevel) && !Trade.GetTakeProfitLevelHit()){
		takeProfitLevelHit = true;
		Trade.SetTakeProfitLevelHit(takeProfitLevelHit);
	}

	// TODO: Bear Check Take Profit Level Hit C-2023-09-08
	if(activePosition && glSellPlaced && (Bar.Low(0) <= normalizedTakeProfitLevel) && !Trade.GetTakeProfitLevelHit()){
		takeProfitLevelHit = true;
		Trade.SetTakeProfitLevelHit(takeProfitLevelHit);
	}

	// Bull Check Stop Loss Level Hit
	if(glBuyPlaced && (Bar.Low(0) <= normalizedStopLossLevel) && !stopLossLevelHit){
		bool success = Trade.DeleteAllPendingOrders(_Symbol);
		if(success){
			stopLossLevelHit = true;
			Trade.SetStopLossLevelHit(stopLossLevelHit);
			glBuyPlaced = false;
		}
	}

	// Bear Check Stop Loss Level Hit
	if(glSellPlaced && (Bar.High(0) >= normalizedStopLossLevel) && !stopLossLevelHit){
		bool success = Trade.DeleteAllPendingOrders(_Symbol);
		if(success){
			stopLossLevelHit = true;
			Trade.SetStopLossLevelHit(stopLossLevelHit);
			glBuyPlaced = false;
		}
	}	


	// Bull Set TP Level(s) 
	if(!takeProfitLevelHit && activePosition && !activeOrders && !stopToTrailingLevelHit && !stopLossLevelHit && glBuyPlaced){
		bool success = false;
		int retryCount = 0;
		int numContracts = 1;
		string debugPrintString;
		string errorDescription;
		normalizedTakeProfitLevel = Trade.GetNormalizedTakeProfitLevel();

		success = Trade.SellLimitWithValidation(_Symbol, numContracts, normalizedTakeProfitLevel);
	}

	// TODO: Bear Set TP Level(s) C-2023-09-08
	if(!takeProfitLevelHit && activePosition && !activeOrders && !stopToTrailingLevelHit && !stopLossLevelHit && glSellPlaced){
		bool success = false;
		int retryCount = 0;
		int numContracts = 1;
		string debugPrintString;
		string errorDescription;
		normalizedTakeProfitLevel = Trade.GetNormalizedTakeProfitLevel();

		success = Trade.BuyLimitWithValidation(_Symbol, numContracts, normalizedTakeProfitLevel);
	}

	// Bull Stop to Scratch
	if(activePosition && glBuyPlaced && !stopToScratchLevelHit){ //Active Position Check Here May Be Uneeded. C-2023-08-26
		if(Bar.High(0) >= normalizedStopToScratchLevel){
			bool success = Trade.AdjustStopToStractch();
			if(success){
				stopToScratchLevelHit = true;
			}
		}
	}

	// Bear Stop to scratch 
	if(activePosition && glSellPlaced && !stopToScratchLevelHit){ //Active Position Check Here May Be Uneeded. C-2023-08-26
		if(Bar.Low(0) <= normalizedStopToScratchLevel){
			bool success = Trade.AdjustStopToStractch();
			if(success){
				stopToScratchLevelHit = true;
			}
		}
	}

	// Bull Stop to Trailing
	if(activePosition && glBuyPlaced && ((Bar.High(0) >= normalizedStopToTrailingLevel) || stopToTrailingLevelHit)){
		// TODO: Really need to create some sort of validation that, instead of checking for 0, checks to see if the level is x% or xPoints away from current price/present stoploss level. C-2023-09-28
		double trailingStopLevel = 0.0;
		trailingStopLevel = TrailingStopFastMovingAverage.Main(glBarShift);
		if(trailingStopLevel == 0){
			TradingUtilities.DebugPrint("Critical Error: Bullish Trailing Stop Level Came Through As: " + trailingStopLevel);
		} else {
			Trade.AdjustBullishTrailingStopLevel(_Symbol, trailingStopLevel, currentBid, symbolStopsLevel);
		}
	} 

	// TODO: Bear Stop to Trailing C-2023-09-08
	if(activePosition && glSellPlaced && ((Bar.Low(0) <= normalizedStopToTrailingLevel) || stopToTrailingLevelHit)){
		// TODO: Really need to create some sort of validation that, instead of checking for 0, checks to see if the level is x% or xPoints away from current price/present stoploss level. C-2023-09-28
		double trailingStopLevel = 0.0;
		trailingStopLevel = TrailingStopFastMovingAverage.Main(glBarShift);
		if(trailingStopLevel == 0){
			TradingUtilities.DebugPrint("Critical Error: Bearish Trailing Stop Level Came Through As: " + trailingStopLevel);
		} else {
			//Trade.AdjustTrailingStopLevel(_Symbol, trailingStopLevel, currentBid, symbolStopsLevel);
			Trade.AdjustBearishTrailingStopLevel(_Symbol, trailingStopLevel, currentAsk, symbolStopsLevel);
		}
	} 
}